import concurrent.futures.thread
from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass
from typing import List
from unittest import mock

import grpc
import grpc_testing
import pytest
from google.protobuf.message import Message as ProtoMessage

from lekko_client.gen.lekko.client.v1beta1.configuration_service_pb2 import DESCRIPTOR


@pytest.fixture
def test_thread():
    client_execution_thread_pool = ThreadPoolExecutor(max_workers=2)

    try:
        yield client_execution_thread_pool
    finally:
        client_execution_thread_pool.shutdown(wait=False)
        client_execution_thread_pool._threads.clear()
        concurrent.futures.thread._threads_queues.clear()


@pytest.fixture
def test_channel(test_thread):
    channel = grpc_testing.channel(DESCRIPTOR.services_by_name.values(), grpc_testing.strict_real_time())
    try:
        with mock.patch("lekko_client.client.get_grpc_channel", return_value=channel):
            yield channel
    finally:
        channel.close()


class MockServer:
    def __init__(self, channel, thread):
        self.channel = channel
        self.thread = thread

    @dataclass
    class MockRequestResponse:
        fn_name: str
        response: ProtoMessage
        status_code = grpc.StatusCode.OK
        error_text: str = ""

    def mock_async_responses(self, mock_requests: List[MockRequestResponse]):
        def server() -> List[ProtoMessage]:
            service = DESCRIPTOR.services_by_name["ConfigurationService"]
            incoming_requests = []
            for mock_request in mock_requests:
                _, request, rpc = self.channel.take_unary_unary(service.methods_by_name[mock_request.fn_name])
                rpc.terminate(mock_request.response, [], mock_request.status_code, mock_request.error_text)
                incoming_requests.append(request)

            return incoming_requests

        server_future = self.thread.submit(server)
        return server_future


@pytest.fixture
def test_server(test_channel, test_thread):
    return MockServer(test_channel, test_thread)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import configuration_service_pb2 as lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2


class ConfigurationServiceStub(object):
    """Initial implementation of a feature flagging service.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetBoolValue = channel.unary_unary(
                '/lekko.client.v1beta1.ConfigurationService/GetBoolValue',
                request_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetBoolValueRequest.SerializeToString,
                response_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetBoolValueResponse.FromString,
                )
        self.GetIntValue = channel.unary_unary(
                '/lekko.client.v1beta1.ConfigurationService/GetIntValue',
                request_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetIntValueRequest.SerializeToString,
                response_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetIntValueResponse.FromString,
                )
        self.GetFloatValue = channel.unary_unary(
                '/lekko.client.v1beta1.ConfigurationService/GetFloatValue',
                request_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetFloatValueRequest.SerializeToString,
                response_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetFloatValueResponse.FromString,
                )
        self.GetStringValue = channel.unary_unary(
                '/lekko.client.v1beta1.ConfigurationService/GetStringValue',
                request_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetStringValueRequest.SerializeToString,
                response_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetStringValueResponse.FromString,
                )
        self.GetProtoValue = channel.unary_unary(
                '/lekko.client.v1beta1.ConfigurationService/GetProtoValue',
                request_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetProtoValueRequest.SerializeToString,
                response_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetProtoValueResponse.FromString,
                )
        self.GetJSONValue = channel.unary_unary(
                '/lekko.client.v1beta1.ConfigurationService/GetJSONValue',
                request_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetJSONValueRequest.SerializeToString,
                response_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetJSONValueResponse.FromString,
                )
        self.Register = channel.unary_unary(
                '/lekko.client.v1beta1.ConfigurationService/Register',
                request_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.RegisterRequest.SerializeToString,
                response_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.RegisterResponse.FromString,
                )
        self.Deregister = channel.unary_unary(
                '/lekko.client.v1beta1.ConfigurationService/Deregister',
                request_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.DeregisterRequest.SerializeToString,
                response_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.DeregisterResponse.FromString,
                )


class ConfigurationServiceServicer(object):
    """Initial implementation of a feature flagging service.
    """

    def GetBoolValue(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIntValue(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetFloatValue(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetStringValue(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetProtoValue(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetJSONValue(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Register(self, request, context):
        """Register is used to denote a RepositoryKey and namespaces within it
        that a client is interested in so the server can cache and keep up to date.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Deregister(self, request, context):
        """Deregister is used to tell the server that a client is shutting down. It is not
        required but preferable to have implementations call this once their lifecycle
        has completed.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ConfigurationServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetBoolValue': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBoolValue,
                    request_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetBoolValueRequest.FromString,
                    response_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetBoolValueResponse.SerializeToString,
            ),
            'GetIntValue': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIntValue,
                    request_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetIntValueRequest.FromString,
                    response_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetIntValueResponse.SerializeToString,
            ),
            'GetFloatValue': grpc.unary_unary_rpc_method_handler(
                    servicer.GetFloatValue,
                    request_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetFloatValueRequest.FromString,
                    response_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetFloatValueResponse.SerializeToString,
            ),
            'GetStringValue': grpc.unary_unary_rpc_method_handler(
                    servicer.GetStringValue,
                    request_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetStringValueRequest.FromString,
                    response_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetStringValueResponse.SerializeToString,
            ),
            'GetProtoValue': grpc.unary_unary_rpc_method_handler(
                    servicer.GetProtoValue,
                    request_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetProtoValueRequest.FromString,
                    response_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetProtoValueResponse.SerializeToString,
            ),
            'GetJSONValue': grpc.unary_unary_rpc_method_handler(
                    servicer.GetJSONValue,
                    request_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetJSONValueRequest.FromString,
                    response_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetJSONValueResponse.SerializeToString,
            ),
            'Register': grpc.unary_unary_rpc_method_handler(
                    servicer.Register,
                    request_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.RegisterRequest.FromString,
                    response_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.RegisterResponse.SerializeToString,
            ),
            'Deregister': grpc.unary_unary_rpc_method_handler(
                    servicer.Deregister,
                    request_deserializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.DeregisterRequest.FromString,
                    response_serializer=lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.DeregisterResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'lekko.client.v1beta1.ConfigurationService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class ConfigurationService(object):
    """Initial implementation of a feature flagging service.
    """

    @staticmethod
    def GetBoolValue(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lekko.client.v1beta1.ConfigurationService/GetBoolValue',
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetBoolValueRequest.SerializeToString,
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetBoolValueResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIntValue(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lekko.client.v1beta1.ConfigurationService/GetIntValue',
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetIntValueRequest.SerializeToString,
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetIntValueResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetFloatValue(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lekko.client.v1beta1.ConfigurationService/GetFloatValue',
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetFloatValueRequest.SerializeToString,
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetFloatValueResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetStringValue(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lekko.client.v1beta1.ConfigurationService/GetStringValue',
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetStringValueRequest.SerializeToString,
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetStringValueResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetProtoValue(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lekko.client.v1beta1.ConfigurationService/GetProtoValue',
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetProtoValueRequest.SerializeToString,
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetProtoValueResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetJSONValue(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lekko.client.v1beta1.ConfigurationService/GetJSONValue',
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetJSONValueRequest.SerializeToString,
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.GetJSONValueResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Register(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lekko.client.v1beta1.ConfigurationService/Register',
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.RegisterRequest.SerializeToString,
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.RegisterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Deregister(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/lekko.client.v1beta1.ConfigurationService/Deregister',
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.DeregisterRequest.SerializeToString,
            lekko_dot_client_dot_v1beta1_dot_configuration__service__pb2.DeregisterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import json
import os
from abc import ABC, abstractmethod
from typing import Any, Dict, Optional, Tuple, Type, TypeVar

import grpc
from google.protobuf import descriptor_pool as proto_descriptor_pool
from google.protobuf import symbol_database as proto_symbol_database
from google.protobuf.any_pb2 import Any as AnyProto
from google.protobuf.message import Message as ProtoMessage

from lekko_client import LEKKO_API_URL, LEKKO_SIDECAR_URL
from lekko_client.exceptions import (
    AuthenticationError,
    FeatureNotFound,
    MismatchedProtoType,
    MismatchedType,
)
from lekko_client.gen.lekko.client.v1beta1.configuration_service_pb2 import (
    GetBoolValueRequest,
    GetFloatValueRequest,
    GetIntValueRequest,
    GetJSONValueRequest,
    GetProtoValueRequest,
    GetStringValueRequest,
    RegisterRequest,
    RepositoryKey,
)
from lekko_client.gen.lekko.client.v1beta1.configuration_service_pb2_grpc import (
    ConfigurationServiceStub,
)
from lekko_client.helpers import convert_context, get_grpc_channel


class Client(ABC):
    ProtoType = TypeVar("ProtoType", bound=ProtoMessage)

    def __init__(
        self,
        owner_name: str,
        repo_name: str,
        namespace: str,
        api_key: Optional[str] = None,
        context: Optional[Dict[str, Any]] = None,
    ):
        self.owner_name = owner_name
        self.repo_name = repo_name
        self.namespace = namespace
        self.context = context or {}

        self.api_key = api_key or os.environ.get("LEKKO_API_KEY")

    @abstractmethod
    def get_bool(self, key: str, context: Dict[str, Any]) -> bool:
        raise NotImplementedError

    @abstractmethod
    def get_int(self, key: str, context: Dict[str, Any]) -> int:
        raise NotImplementedError

    @abstractmethod
    def get_float(self, key: str, context: Dict[str, Any]) -> float:
        raise NotImplementedError

    @abstractmethod
    def get_string(self, key: str, context: Dict[str, Any]) -> str:
        raise NotImplementedError

    @abstractmethod
    def get_json(self, key: str, context: Dict[str, Any]) -> dict:
        pass

    @abstractmethod
    def get_proto(
        self,
        key: str,
        context: Dict[str, Any],
    ) -> ProtoMessage:
        pass

    @abstractmethod
    def get_proto_by_type(
        self,
        key: str,
        context: Dict[str, Any],
        proto_message_type: Type[ProtoType],
    ) -> ProtoType:
        pass


class GRPCClient(Client):
    _channels: Dict[Tuple[str, str], grpc.Channel] = {}

    class JsonBytes(bytes):
        pass

    ReturnType = TypeVar("ReturnType", str, float, int, bool, dict, AnyProto, JsonBytes)

    _TYPE_MAPPING: Dict[Type, Tuple[str, Type]] = {
        bool: ("GetBoolValue", GetBoolValueRequest),
        int: ("GetIntValue", GetIntValueRequest),
        str: ("GetStringValue", GetStringValueRequest),
        float: ("GetFloatValue", GetFloatValueRequest),
        JsonBytes: ("GetJSONValue", GetJSONValueRequest),
        AnyProto: ("GetProtoValue", GetProtoValueRequest),
    }

    def __init__(
        self,
        uri: str,
        owner_name: str,
        repo_name: str,
        namespace: str,
        api_key: Optional[str] = None,
        context: Optional[Dict[str, Any]] = None,
        credentials: Optional[grpc.ChannelCredentials] = None,
    ):
        super().__init__(owner_name, repo_name, namespace, api_key)
        self.repository = RepositoryKey(owner_name=owner_name, repo_name=repo_name)
        self.api_key = api_key or os.environ.get("LEKKO_API_KEY")

        self.namespace = namespace
        self.context = context or {}
        self.uri = uri

        if not self.api_key:
            raise AuthenticationError("Must provide API key and URI")

        channel = get_grpc_channel(self.uri, self.api_key, credentials)

        self._client = ConfigurationServiceStub(channel)
        try:
            self._client.Register(RegisterRequest(repo_key=self.repository, namespace_list=[namespace]))
        except grpc.RpcError:
            # TODO:SAM - re-registering shouldn't cause errors in the future
            pass

    def get_bool(self, key: str, context: Dict[str, Any]) -> bool:
        return self._get(key, context, bool)

    def get_int(self, key: str, context: Dict[str, Any]) -> int:
        return self._get(key, context, int)

    def get_float(self, key: str, context: Dict[str, Any]) -> float:
        return self._get(key, context, float)

    def get_string(self, key: str, context: Dict[str, Any]) -> str:
        return self._get(key, context, str)

    def get_json(self, key: str, context: Dict[str, Any]) -> dict:
        json_bytes = self._get(key, context, GRPCClient.JsonBytes)
        return json.loads(json_bytes.decode("utf-8"))

    def get_proto(self, key: str, context: Dict[str, Any]) -> ProtoMessage:
        val = self._get(key, context, AnyProto)
        db = proto_symbol_database.SymbolDatabase(pool=proto_descriptor_pool.Default())
        try:
            ret_val = db.GetSymbol(val.type_url.split("/")[1])()
            if val.Unpack(ret_val):
                return ret_val
        except (KeyError, IndexError):
            pass
        return val

    def get_proto_by_type(
        self,
        key: str,
        context: Dict[str, Any],
        proto_message_type: Type[Client.ProtoType],
    ) -> Client.ProtoType:
        val = self._get(key, context, AnyProto)
        ret_val = proto_message_type()
        if val.Unpack(ret_val):
            return ret_val

        raise MismatchedProtoType(f"Error unpacking from {val.type_url} to {proto_message_type.DESCRIPTOR.name}")

    def _get(self, key: str, context: Dict[str, Any], typ: Type[ReturnType]) -> ReturnType:
        ctx = self.context | context
        fn_name, req_type = self._TYPE_MAPPING[typ]
        try:
            req = req_type(
                key=key,
                context=convert_context(ctx),
                namespace=self.namespace,
                repo_key=self.repository,
            )
            response = getattr(self._client, fn_name)(req)
            return response.value
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.INVALID_ARGUMENT:
                if "type mismatch" in (e.details() or ""):
                    raise MismatchedType(e.details()) from e
                elif "not found" in (e.details() or ""):
                    raise FeatureNotFound(e.details()) from e
            raise


class SidecarClient(GRPCClient):
    def __init__(
        self,
        owner_name: str,
        repo_name: str,
        namespace: str,
        api_key: Optional[str] = None,
        context: Optional[Dict[str, Any]] = None,
        uri: str = LEKKO_SIDECAR_URL,
    ):
        super().__init__(uri, owner_name, repo_name, namespace, api_key, context)


class APIClient(GRPCClient):
    def __init__(
        self,
        owner_name: str,
        repo_name: str,
        namespace: str,
        api_key: Optional[str] = None,
        context: Optional[Dict[str, Any]] = None,
        uri: str = LEKKO_API_URL,
        credentials: grpc.ChannelCredentials = grpc.ssl_channel_credentials(),
    ):
        super().__init__(
            uri,
            owner_name,
            repo_name,
            namespace,
            api_key,
            context,
            credentials,
        )



import json

from lekko_client.client import SidecarClient
from lekko_client.gen.lekko.client.v1beta1 import configuration_service_pb2 as messages


def test_get_bool(test_server):
    requests = [
        test_server.MockRequestResponse("Register", messages.RegisterResponse),
        test_server.MockRequestResponse("GetBoolValue", messages.GetBoolValueResponse(value=True)),
    ]

    test_server.mock_async_responses(requests)

    client = SidecarClient("owner", "repo", "namespace", "lekko_apikey123")
    resp = client.get_bool("val", {"context": True})

    assert resp is True


def test_get_int(test_server):
    requests = [
        test_server.MockRequestResponse("Register", messages.RegisterResponse),
        test_server.MockRequestResponse("GetIntValue", messages.GetIntValueResponse(value=10)),
    ]

    test_server.mock_async_responses(requests)

    client = SidecarClient("owner", "repo", "namespace", "lekko_apikey123")
    resp = client.get_int("val", {"context": 5})

    assert resp == 10


def test_get_float(test_server):
    requests = [
        test_server.MockRequestResponse("Register", messages.RegisterResponse),
        test_server.MockRequestResponse("GetFloatValue", messages.GetFloatValueResponse(value=1.5)),
    ]

    test_server.mock_async_responses(requests)

    client = SidecarClient("owner", "repo", "namespace", "lekko_apikey123")
    resp = client.get_float("val", {"context": 2.5})

    assert resp == 1.5


def test_get_string(test_server):
    requests = [
        test_server.MockRequestResponse("Register", messages.RegisterResponse),
        test_server.MockRequestResponse("GetStringValue", messages.GetStringValueResponse(value="feature value")),
    ]
    test_server.mock_async_responses(requests)

    client = SidecarClient("owner", "repo", "namespace", "lekko_apikey123")
    resp = client.get_string("val", {"conext": "hello"})

    assert resp == "feature value"


def test_get_json(test_server):
    expected = {"key": "value", "int_key": 1}
    requests = [
        test_server.MockRequestResponse("Register", messages.RegisterResponse),
        test_server.MockRequestResponse(
            "GetJSONValue", messages.GetJSONValueResponse(value=bytes(json.dumps(expected), "utf-8"))
        ),
    ]
    test_server.mock_async_responses(requests)

    client = SidecarClient("owner", "repo", "namespace", "lekko_apikey123")
    resp = client.get_json("val", {})

    assert resp == expected



from typing import Any, Dict, Optional, Tuple

import grpc
from grpc_interceptor import ClientCallDetails, ClientInterceptor

from lekko_client.gen.lekko.client.v1beta1.configuration_service_pb2 import Value


def convert_context(context: dict) -> Dict[str, Value]:
    def convert_value(val: Any) -> Value:
        if isinstance(val, bool):
            return Value(bool_value=val)
        elif isinstance(val, int):
            return Value(int_value=val)
        elif isinstance(val, float):
            return Value(double_value=val)
        else:
            return Value(string_value=str(val))

    return {k: convert_value(v) for k, v in context.items()}


class ApiKeyInterceptor(ClientInterceptor):
    """A test interceptor that injects invocation metadata."""

    def __init__(self, api_key):
        self.api_key = api_key

    def intercept(self, method, request_or_iterator, call_details):
        new_details = ClientCallDetails(
            call_details.method,
            call_details.timeout,
            [("apikey", self.api_key)],
            call_details.credentials,
            call_details.wait_for_ready,
            call_details.compression,
        )

        return method(request_or_iterator, new_details)


_CHANNELS: Dict[Tuple[str, str], grpc.Channel] = {}


def get_grpc_channel(url: str, api_key: str, credentials: Optional[grpc.ChannelCredentials] = None) -> grpc.Channel:
    if (url, api_key) not in _CHANNELS:
        if credentials:
            channel = grpc.secure_channel(url, credentials)
        else:
            channel = grpc.insecure_channel(url)

        channel = grpc.intercept_channel(channel, *[ApiKeyInterceptor(api_key)])
        _CHANNELS[(url, api_key)] = channel

    return _CHANNELS[(url, api_key)]



class LekkoError(Exception):
    pass


class AuthenticationError(LekkoError):
    pass


class LekkoRpcError(LekkoError):
    pass


class FeatureNotFound(LekkoRpcError):
    pass


class MismatchedType(LekkoRpcError):
    pass


class MismatchedProtoType(LekkoError):
    pass



"""Lekko Python SDK Client"""

__version__ = "0.1.1"

LEKKO_API_URL = "prod.api.lekko.dev:443"
LEKKO_SIDECAR_URL = "localhost:50051"

from lekko_client.client import APIClient, SidecarClient  # noqa
from lekko_client.exceptions import *  # noqa



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2022 Lekko Technologies, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import builtins
import collections.abc
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class RepositoryKey(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OWNER_NAME_FIELD_NUMBER: builtins.int
    REPO_NAME_FIELD_NUMBER: builtins.int
    owner_name: builtins.str
    repo_name: builtins.str
    def __init__(
        self,
        *,
        owner_name: builtins.str = ...,
        repo_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["owner_name", b"owner_name", "repo_name", b"repo_name"]) -> None: ...

global___RepositoryKey = RepositoryKey

@typing_extensions.final
class GetBoolValueRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ContextEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___Value: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___Value | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    KEY_FIELD_NUMBER: builtins.int
    CONTEXT_FIELD_NUMBER: builtins.int
    NAMESPACE_FIELD_NUMBER: builtins.int
    REPO_KEY_FIELD_NUMBER: builtins.int
    key: builtins.str
    @property
    def context(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___Value]: ...
    namespace: builtins.str
    @property
    def repo_key(self) -> global___RepositoryKey: ...
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        context: collections.abc.Mapping[builtins.str, global___Value] | None = ...,
        namespace: builtins.str = ...,
        repo_key: global___RepositoryKey | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["repo_key", b"repo_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["context", b"context", "key", b"key", "namespace", b"namespace", "repo_key", b"repo_key"]) -> None: ...

global___GetBoolValueRequest = GetBoolValueRequest

@typing_extensions.final
class GetBoolValueResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.bool
    def __init__(
        self,
        *,
        value: builtins.bool = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value", b"value"]) -> None: ...

global___GetBoolValueResponse = GetBoolValueResponse

@typing_extensions.final
class GetIntValueRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ContextEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___Value: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___Value | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    KEY_FIELD_NUMBER: builtins.int
    CONTEXT_FIELD_NUMBER: builtins.int
    NAMESPACE_FIELD_NUMBER: builtins.int
    REPO_KEY_FIELD_NUMBER: builtins.int
    key: builtins.str
    @property
    def context(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___Value]: ...
    namespace: builtins.str
    @property
    def repo_key(self) -> global___RepositoryKey: ...
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        context: collections.abc.Mapping[builtins.str, global___Value] | None = ...,
        namespace: builtins.str = ...,
        repo_key: global___RepositoryKey | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["repo_key", b"repo_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["context", b"context", "key", b"key", "namespace", b"namespace", "repo_key", b"repo_key"]) -> None: ...

global___GetIntValueRequest = GetIntValueRequest

@typing_extensions.final
class GetIntValueResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.int
    def __init__(
        self,
        *,
        value: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value", b"value"]) -> None: ...

global___GetIntValueResponse = GetIntValueResponse

@typing_extensions.final
class GetFloatValueRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ContextEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___Value: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___Value | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    KEY_FIELD_NUMBER: builtins.int
    CONTEXT_FIELD_NUMBER: builtins.int
    NAMESPACE_FIELD_NUMBER: builtins.int
    REPO_KEY_FIELD_NUMBER: builtins.int
    key: builtins.str
    @property
    def context(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___Value]: ...
    namespace: builtins.str
    @property
    def repo_key(self) -> global___RepositoryKey: ...
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        context: collections.abc.Mapping[builtins.str, global___Value] | None = ...,
        namespace: builtins.str = ...,
        repo_key: global___RepositoryKey | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["repo_key", b"repo_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["context", b"context", "key", b"key", "namespace", b"namespace", "repo_key", b"repo_key"]) -> None: ...

global___GetFloatValueRequest = GetFloatValueRequest

@typing_extensions.final
class GetFloatValueResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.float
    def __init__(
        self,
        *,
        value: builtins.float = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value", b"value"]) -> None: ...

global___GetFloatValueResponse = GetFloatValueResponse

@typing_extensions.final
class GetStringValueRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ContextEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___Value: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___Value | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    KEY_FIELD_NUMBER: builtins.int
    CONTEXT_FIELD_NUMBER: builtins.int
    NAMESPACE_FIELD_NUMBER: builtins.int
    REPO_KEY_FIELD_NUMBER: builtins.int
    key: builtins.str
    @property
    def context(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___Value]: ...
    namespace: builtins.str
    @property
    def repo_key(self) -> global___RepositoryKey: ...
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        context: collections.abc.Mapping[builtins.str, global___Value] | None = ...,
        namespace: builtins.str = ...,
        repo_key: global___RepositoryKey | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["repo_key", b"repo_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["context", b"context", "key", b"key", "namespace", b"namespace", "repo_key", b"repo_key"]) -> None: ...

global___GetStringValueRequest = GetStringValueRequest

@typing_extensions.final
class GetStringValueResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.str
    def __init__(
        self,
        *,
        value: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value", b"value"]) -> None: ...

global___GetStringValueResponse = GetStringValueResponse

@typing_extensions.final
class GetProtoValueRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ContextEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___Value: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___Value | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    KEY_FIELD_NUMBER: builtins.int
    CONTEXT_FIELD_NUMBER: builtins.int
    NAMESPACE_FIELD_NUMBER: builtins.int
    REPO_KEY_FIELD_NUMBER: builtins.int
    key: builtins.str
    @property
    def context(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___Value]: ...
    namespace: builtins.str
    @property
    def repo_key(self) -> global___RepositoryKey: ...
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        context: collections.abc.Mapping[builtins.str, global___Value] | None = ...,
        namespace: builtins.str = ...,
        repo_key: global___RepositoryKey | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["repo_key", b"repo_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["context", b"context", "key", b"key", "namespace", b"namespace", "repo_key", b"repo_key"]) -> None: ...

global___GetProtoValueRequest = GetProtoValueRequest

@typing_extensions.final
class GetProtoValueResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    @property
    def value(self) -> google.protobuf.any_pb2.Any: ...
    def __init__(
        self,
        *,
        value: google.protobuf.any_pb2.Any | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["value", b"value"]) -> None: ...

global___GetProtoValueResponse = GetProtoValueResponse

@typing_extensions.final
class GetJSONValueRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing_extensions.final
    class ContextEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___Value: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___Value | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    KEY_FIELD_NUMBER: builtins.int
    CONTEXT_FIELD_NUMBER: builtins.int
    NAMESPACE_FIELD_NUMBER: builtins.int
    REPO_KEY_FIELD_NUMBER: builtins.int
    key: builtins.str
    @property
    def context(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___Value]: ...
    namespace: builtins.str
    @property
    def repo_key(self) -> global___RepositoryKey: ...
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        context: collections.abc.Mapping[builtins.str, global___Value] | None = ...,
        namespace: builtins.str = ...,
        repo_key: global___RepositoryKey | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["repo_key", b"repo_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["context", b"context", "key", b"key", "namespace", b"namespace", "repo_key", b"repo_key"]) -> None: ...

global___GetJSONValueRequest = GetJSONValueRequest

@typing_extensions.final
class GetJSONValueResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VALUE_FIELD_NUMBER: builtins.int
    value: builtins.bytes
    def __init__(
        self,
        *,
        value: builtins.bytes = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["value", b"value"]) -> None: ...

global___GetJSONValueResponse = GetJSONValueResponse

@typing_extensions.final
class Value(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BOOL_VALUE_FIELD_NUMBER: builtins.int
    INT_VALUE_FIELD_NUMBER: builtins.int
    DOUBLE_VALUE_FIELD_NUMBER: builtins.int
    STRING_VALUE_FIELD_NUMBER: builtins.int
    bool_value: builtins.bool
    int_value: builtins.int
    double_value: builtins.float
    string_value: builtins.str
    def __init__(
        self,
        *,
        bool_value: builtins.bool = ...,
        int_value: builtins.int = ...,
        double_value: builtins.float = ...,
        string_value: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["bool_value", b"bool_value", "double_value", b"double_value", "int_value", b"int_value", "kind", b"kind", "string_value", b"string_value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["bool_value", b"bool_value", "double_value", b"double_value", "int_value", b"int_value", "kind", b"kind", "string_value", b"string_value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["kind", b"kind"]) -> typing_extensions.Literal["bool_value", "int_value", "double_value", "string_value"] | None: ...

global___Value = Value

@typing_extensions.final
class RegisterRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REPO_KEY_FIELD_NUMBER: builtins.int
    NAMESPACE_LIST_FIELD_NUMBER: builtins.int
    @property
    def repo_key(self) -> global___RepositoryKey: ...
    @property
    def namespace_list(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """The namespaces to register within the repo. If empty,
        all namespaces will be registered.
        """
    def __init__(
        self,
        *,
        repo_key: global___RepositoryKey | None = ...,
        namespace_list: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["repo_key", b"repo_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["namespace_list", b"namespace_list", "repo_key", b"repo_key"]) -> None: ...

global___RegisterRequest = RegisterRequest

@typing_extensions.final
class RegisterResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___RegisterResponse = RegisterResponse

@typing_extensions.final
class DeregisterRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeregisterRequest = DeregisterRequest

@typing_extensions.final
class DeregisterResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___DeregisterResponse = DeregisterResponse



import argparse

import grpc
from google.protobuf.json_format import MessageToDict

from lekko_client import APIClient, SidecarClient
from lekko_client.exceptions import LekkoError

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--apikey", type=str)
    parser.add_argument("--sidecar", action="store_true")
    parser.add_argument("--owner", type=str)
    parser.add_argument("--repo", type=str)
    parser.add_argument("--namespace", type=str, default="default")
    parser.add_argument("--feature", type=str, default="example")
    parser.add_argument(
        "--feature-type",
        type=str,
        choices=["bool", "int", "float", "str", "json", "proto"],
        default="bool",
    )
    parser.add_argument("--proto-type", type=str, default="")
    parser.add_argument("--proto-file", type=str)
    args = parser.parse_args()

    client_cls = SidecarClient if args.sidecar else APIClient
    client = client_cls(args.owner, args.repo, args.namespace, api_key=args.apikey)

    val = None
    try:
        if args.feature_type == "bool":
            val = client.get_bool(args.feature, {})
        elif args.feature_type == "int":
            val = client.get_int(args.feature, {})
        elif args.feature_type == "str":
            val = client.get_string(args.feature, {})
        elif args.feature_type == "json":
            val = client.get_json(args.feature, {})
        elif args.feature_type == "float":
            val = client.get_float(args.feature, {})
        elif args.feature_type == "proto":
            if not args.proto_file:
                print(
                    "Must provide a --proto-file. Could be a path or a well-known proto like 'google/protobuf/wrappers.proto'"
                )
            else:
                imported_proto = grpc.protos(args.proto_file)

                if args.proto_type:
                    msg_type = getattr(imported_proto, args.proto_type)
                    val = MessageToDict(client.get_proto_by_type(args.feature, {}, msg_type))
                else:
                    val = MessageToDict(client.get_proto(args.feature, {}))
        print(f"Got {val} for feature {args.namespace}/{args.feature}")
    except LekkoError as e:
        print(f"Failed to get feature: {e}")
        if e.__cause__:
            print(f"Caused by: {e.__cause__}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: lekko/client/v1beta1/configuration_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0lekko/client/v1beta1/configuration_service.proto\x12\x14lekko.client.v1beta1\x1a\x19google/protobuf/any.proto\"K\n\rRepositoryKey\x12\x1d\n\nowner_name\x18\x01 \x01(\tR\townerName\x12\x1b\n\trepo_name\x18\x02 \x01(\tR\x08repoName\"\xb0\x02\n\x13GetBoolValueRequest\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12P\n\x07\x63ontext\x18\x02 \x03(\x0b\x32\x36.lekko.client.v1beta1.GetBoolValueRequest.ContextEntryR\x07\x63ontext\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12>\n\x08repo_key\x18\x04 \x01(\x0b\x32#.lekko.client.v1beta1.RepositoryKeyR\x07repoKey\x1aW\n\x0c\x43ontextEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.lekko.client.v1beta1.ValueR\x05value:\x02\x38\x01\",\n\x14GetBoolValueResponse\x12\x14\n\x05value\x18\x01 \x01(\x08R\x05value\"\xae\x02\n\x12GetIntValueRequest\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12O\n\x07\x63ontext\x18\x02 \x03(\x0b\x32\x35.lekko.client.v1beta1.GetIntValueRequest.ContextEntryR\x07\x63ontext\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12>\n\x08repo_key\x18\x04 \x01(\x0b\x32#.lekko.client.v1beta1.RepositoryKeyR\x07repoKey\x1aW\n\x0c\x43ontextEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.lekko.client.v1beta1.ValueR\x05value:\x02\x38\x01\"+\n\x13GetIntValueResponse\x12\x14\n\x05value\x18\x01 \x01(\x03R\x05value\"\xb2\x02\n\x14GetFloatValueRequest\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12Q\n\x07\x63ontext\x18\x02 \x03(\x0b\x32\x37.lekko.client.v1beta1.GetFloatValueRequest.ContextEntryR\x07\x63ontext\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12>\n\x08repo_key\x18\x04 \x01(\x0b\x32#.lekko.client.v1beta1.RepositoryKeyR\x07repoKey\x1aW\n\x0c\x43ontextEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.lekko.client.v1beta1.ValueR\x05value:\x02\x38\x01\"-\n\x15GetFloatValueResponse\x12\x14\n\x05value\x18\x01 \x01(\x01R\x05value\"\xb4\x02\n\x15GetStringValueRequest\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12R\n\x07\x63ontext\x18\x02 \x03(\x0b\x32\x38.lekko.client.v1beta1.GetStringValueRequest.ContextEntryR\x07\x63ontext\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12>\n\x08repo_key\x18\x04 \x01(\x0b\x32#.lekko.client.v1beta1.RepositoryKeyR\x07repoKey\x1aW\n\x0c\x43ontextEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.lekko.client.v1beta1.ValueR\x05value:\x02\x38\x01\".\n\x16GetStringValueResponse\x12\x14\n\x05value\x18\x01 \x01(\tR\x05value\"\xb2\x02\n\x14GetProtoValueRequest\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12Q\n\x07\x63ontext\x18\x02 \x03(\x0b\x32\x37.lekko.client.v1beta1.GetProtoValueRequest.ContextEntryR\x07\x63ontext\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12>\n\x08repo_key\x18\x04 \x01(\x0b\x32#.lekko.client.v1beta1.RepositoryKeyR\x07repoKey\x1aW\n\x0c\x43ontextEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.lekko.client.v1beta1.ValueR\x05value:\x02\x38\x01\"C\n\x15GetProtoValueResponse\x12*\n\x05value\x18\x01 \x01(\x0b\x32\x14.google.protobuf.AnyR\x05value\"\xb0\x02\n\x13GetJSONValueRequest\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12P\n\x07\x63ontext\x18\x02 \x03(\x0b\x32\x36.lekko.client.v1beta1.GetJSONValueRequest.ContextEntryR\x07\x63ontext\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12>\n\x08repo_key\x18\x04 \x01(\x0b\x32#.lekko.client.v1beta1.RepositoryKeyR\x07repoKey\x1aW\n\x0c\x43ontextEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x31\n\x05value\x18\x02 \x01(\x0b\x32\x1b.lekko.client.v1beta1.ValueR\x05value:\x02\x38\x01\",\n\x14GetJSONValueResponse\x12\x14\n\x05value\x18\x01 \x01(\x0cR\x05value\"\x99\x01\n\x05Value\x12\x1f\n\nbool_value\x18\x01 \x01(\x08H\x00R\tboolValue\x12\x1d\n\tint_value\x18\x02 \x01(\x03H\x00R\x08intValue\x12#\n\x0c\x64ouble_value\x18\x03 \x01(\x01H\x00R\x0b\x64oubleValue\x12#\n\x0cstring_value\x18\x04 \x01(\tH\x00R\x0bstringValueB\x06\n\x04kind\"x\n\x0fRegisterRequest\x12>\n\x08repo_key\x18\x01 \x01(\x0b\x32#.lekko.client.v1beta1.RepositoryKeyR\x07repoKey\x12%\n\x0enamespace_list\x18\x02 \x03(\tR\rnamespaceList\"\x12\n\x10RegisterResponse\"\x13\n\x11\x44\x65registerRequest\"\x14\n\x12\x44\x65registerResponse2\xd5\x06\n\x14\x43onfigurationService\x12g\n\x0cGetBoolValue\x12).lekko.client.v1beta1.GetBoolValueRequest\x1a*.lekko.client.v1beta1.GetBoolValueResponse\"\x00\x12\x64\n\x0bGetIntValue\x12(.lekko.client.v1beta1.GetIntValueRequest\x1a).lekko.client.v1beta1.GetIntValueResponse\"\x00\x12j\n\rGetFloatValue\x12*.lekko.client.v1beta1.GetFloatValueRequest\x1a+.lekko.client.v1beta1.GetFloatValueResponse\"\x00\x12m\n\x0eGetStringValue\x12+.lekko.client.v1beta1.GetStringValueRequest\x1a,.lekko.client.v1beta1.GetStringValueResponse\"\x00\x12j\n\rGetProtoValue\x12*.lekko.client.v1beta1.GetProtoValueRequest\x1a+.lekko.client.v1beta1.GetProtoValueResponse\"\x00\x12g\n\x0cGetJSONValue\x12).lekko.client.v1beta1.GetJSONValueRequest\x1a*.lekko.client.v1beta1.GetJSONValueResponse\"\x00\x12[\n\x08Register\x12%.lekko.client.v1beta1.RegisterRequest\x1a&.lekko.client.v1beta1.RegisterResponse\"\x00\x12\x61\n\nDeregister\x12\'.lekko.client.v1beta1.DeregisterRequest\x1a(.lekko.client.v1beta1.DeregisterResponse\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'lekko.client.v1beta1.configuration_service_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _GETBOOLVALUEREQUEST_CONTEXTENTRY._options = None
  _GETBOOLVALUEREQUEST_CONTEXTENTRY._serialized_options = b'8\001'
  _GETINTVALUEREQUEST_CONTEXTENTRY._options = None
  _GETINTVALUEREQUEST_CONTEXTENTRY._serialized_options = b'8\001'
  _GETFLOATVALUEREQUEST_CONTEXTENTRY._options = None
  _GETFLOATVALUEREQUEST_CONTEXTENTRY._serialized_options = b'8\001'
  _GETSTRINGVALUEREQUEST_CONTEXTENTRY._options = None
  _GETSTRINGVALUEREQUEST_CONTEXTENTRY._serialized_options = b'8\001'
  _GETPROTOVALUEREQUEST_CONTEXTENTRY._options = None
  _GETPROTOVALUEREQUEST_CONTEXTENTRY._serialized_options = b'8\001'
  _GETJSONVALUEREQUEST_CONTEXTENTRY._options = None
  _GETJSONVALUEREQUEST_CONTEXTENTRY._serialized_options = b'8\001'
  _globals['_REPOSITORYKEY']._serialized_start=101
  _globals['_REPOSITORYKEY']._serialized_end=176
  _globals['_GETBOOLVALUEREQUEST']._serialized_start=179
  _globals['_GETBOOLVALUEREQUEST']._serialized_end=483
  _globals['_GETBOOLVALUEREQUEST_CONTEXTENTRY']._serialized_start=396
  _globals['_GETBOOLVALUEREQUEST_CONTEXTENTRY']._serialized_end=483
  _globals['_GETBOOLVALUERESPONSE']._serialized_start=485
  _globals['_GETBOOLVALUERESPONSE']._serialized_end=529
  _globals['_GETINTVALUEREQUEST']._serialized_start=532
  _globals['_GETINTVALUEREQUEST']._serialized_end=834
  _globals['_GETINTVALUEREQUEST_CONTEXTENTRY']._serialized_start=396
  _globals['_GETINTVALUEREQUEST_CONTEXTENTRY']._serialized_end=483
  _globals['_GETINTVALUERESPONSE']._serialized_start=836
  _globals['_GETINTVALUERESPONSE']._serialized_end=879
  _globals['_GETFLOATVALUEREQUEST']._serialized_start=882
  _globals['_GETFLOATVALUEREQUEST']._serialized_end=1188
  _globals['_GETFLOATVALUEREQUEST_CONTEXTENTRY']._serialized_start=396
  _globals['_GETFLOATVALUEREQUEST_CONTEXTENTRY']._serialized_end=483
  _globals['_GETFLOATVALUERESPONSE']._serialized_start=1190
  _globals['_GETFLOATVALUERESPONSE']._serialized_end=1235
  _globals['_GETSTRINGVALUEREQUEST']._serialized_start=1238
  _globals['_GETSTRINGVALUEREQUEST']._serialized_end=1546
  _globals['_GETSTRINGVALUEREQUEST_CONTEXTENTRY']._serialized_start=396
  _globals['_GETSTRINGVALUEREQUEST_CONTEXTENTRY']._serialized_end=483
  _globals['_GETSTRINGVALUERESPONSE']._serialized_start=1548
  _globals['_GETSTRINGVALUERESPONSE']._serialized_end=1594
  _globals['_GETPROTOVALUEREQUEST']._serialized_start=1597
  _globals['_GETPROTOVALUEREQUEST']._serialized_end=1903
  _globals['_GETPROTOVALUEREQUEST_CONTEXTENTRY']._serialized_start=396
  _globals['_GETPROTOVALUEREQUEST_CONTEXTENTRY']._serialized_end=483
  _globals['_GETPROTOVALUERESPONSE']._serialized_start=1905
  _globals['_GETPROTOVALUERESPONSE']._serialized_end=1972
  _globals['_GETJSONVALUEREQUEST']._serialized_start=1975
  _globals['_GETJSONVALUEREQUEST']._serialized_end=2279
  _globals['_GETJSONVALUEREQUEST_CONTEXTENTRY']._serialized_start=396
  _globals['_GETJSONVALUEREQUEST_CONTEXTENTRY']._serialized_end=483
  _globals['_GETJSONVALUERESPONSE']._serialized_start=2281
  _globals['_GETJSONVALUERESPONSE']._serialized_end=2325
  _globals['_VALUE']._serialized_start=2328
  _globals['_VALUE']._serialized_end=2481
  _globals['_REGISTERREQUEST']._serialized_start=2483
  _globals['_REGISTERREQUEST']._serialized_end=2603
  _globals['_REGISTERRESPONSE']._serialized_start=2605
  _globals['_REGISTERRESPONSE']._serialized_end=2623
  _globals['_DEREGISTERREQUEST']._serialized_start=2625
  _globals['_DEREGISTERREQUEST']._serialized_end=2644
  _globals['_DEREGISTERRESPONSE']._serialized_start=2646
  _globals['_DEREGISTERRESPONSE']._serialized_end=2666
  _globals['_CONFIGURATIONSERVICE']._serialized_start=2669
  _globals['_CONFIGURATIONSERVICE']._serialized_end=3522
# @@protoc_insertion_point(module_scope)



import grpc

from lekko_client.gen.lekko.client.v1beta1.configuration_service_pb2 import (
    GetBoolValueResponse,
    GetFloatValueResponse,
    GetIntValueResponse,
    GetJSONValueResponse,
    GetProtoValueResponse,
    GetStringValueResponse,
    RegisterResponse,
)
from lekko_client.gen.lekko.client.v1beta1.configuration_service_pb2_grpc import (
    ConfigurationServiceServicer,
)


class MockConfigurationService(ConfigurationServiceServicer):
    RESPONSES = {
        "bool_val": True,
        "int_val": 10,
        "float_val": 25.5,
        "string_val": "hello world",
        "json_val": {"key": "value"},
    }

    def _get(self, request, context, expected_type, resp_type):
        val = self.RESPONSES.get(request.key)
        if not val:
            context.set_code(grpc.StatusCode.INVALID_ARGUMENT)
            context.set_details("not found")
            return resp_type()
        elif not isinstance(val, expected_type):
            context.set_code(grpc.StatusCode.INVALID_ARGUMENT)
            context.set_details("type mismatch")
            return resp_type()

        return resp_type(value=val)

    def GetBoolValue(self, request, context):
        return self._get(request, context, bool, GetBoolValueResponse)

    def GetIntValue(self, request, context):
        return self._get(request, context, int, GetIntValueResponse)

    def GetFloatValue(self, request, context):
        return self._get(request, context, float, GetFloatValueResponse)

    def GetStringValue(self, request, context):
        return self._get(request, context, str, GetStringValueResponse)

    def GetProtoValue(self, request, context):
        return GetProtoValueResponse()

    def GetJSONValue(self, request, context):
        return self._get(request, context, dict, GetJSONValueResponse)

    def Register(self, request, context):
        return RegisterResponse()
